import 'dart:core';
import 'package:latlong2/latlong.dart';
import 'package:flutter/material.dart';

class Measurement {
  int measurementID;
  DateTime timeMeasured;
  Sensor sensor;
  String bodyOfWater;
  LatLng center;
  List <SubDiv> subDivs;

  Measurement({
    required this.measurementID,
    required this.timeMeasured,
    required this.sensor,
    required this.bodyOfWater,
    required this.center,
    required this.subDivs,
  });

  factory Measurement.fromJson(Map<String, dynamic> json) {
    return Measurement(
        measurementID: json['MeasurementID'],
        timeMeasured: DateTime.parse(json['TimeMeasured']),
        sensor: Sensor.fromJson(json['Sensor']),
        bodyOfWater: json['BodyOfWater'] ?? 'nil',
        center: LatLng(json['CenterLat'], json['CenterLon']),
        subDivs: (json['Subdivisions'] as List<dynamic>).map((data) => SubDiv.fromJson(data)).toList(),
    );
  }
}

class SubDiv {
  String sub_div_id;
  int groupID;
  double minThickness;
  double avgThickness;
  LatLng center;
  double accuracy;
  Color color;
  List<IceStats> iceStats;

  SubDiv({
    required this.sub_div_id,
    required this.groupID,
    required this.minThickness,
    required this.avgThickness,
    required this.center,
    required this.accuracy,
    required this.color,
    required this.iceStats,
  });

  factory SubDiv.fromJson(Map<String, dynamic> json) {
    return SubDiv(
      sub_div_id: json['SubdivID'].toString(),
      groupID: json['GroupID'],
      minThickness: json['MinThickness'],
      avgThickness: json['AvgThickness'],
      center: LatLng(json['CenLatitude'], json['CenLongitude']),
      accuracy: json['Accuracy'],
      // Set grey as default color
      color: json['Color'] != null ? Color(json['Color']) : Colors.grey,
      iceStats: (json['IceStats'] as List<dynamic>?)
          ?.map((data) => IceStats.fromJson(data))
          .toList() ?? [],
    );
  }
}

class IceStats {
  DateTime dateTime;
  double slushIce;
  double blackIce;
  double totalIce;
  double snowDepth;
  double totalSnow;
  double cloudCover;
  double temperature;

  IceStats({
    required this.dateTime,
    required this.slushIce,
    required this.blackIce,
    required this.totalIce,
    required this.snowDepth,
    required this.totalSnow,
    required this.cloudCover,
    required this.temperature,
  });

  factory IceStats.fromJson(Map<String, dynamic>? json) {
    if (json == null) { // Return empty json
      return IceStats(
        dateTime: DateTime.now(),
        slushIce: 0.0,
        blackIce: 0.0,
        totalIce: 0.0,
        snowDepth: 0.0,
        totalSnow: 0.0,
        cloudCover: 0.0,
        temperature: 0.0,
      );
    }

    return IceStats(
      dateTime: DateTime.parse(json['Date']),
      slushIce: json['Slush ice (m)'] != null ? json['Slush ice (m)'].toDouble() : 0.0,
      blackIce: json['Black ice (m)'] != null ? json['Black ice (m)'].toDouble() : 0.0,
      totalIce: json['Total ice (m)'] != null ? json['Total ice (m)'].toDouble() : 0.0,
      snowDepth: json['Snow depth (m)'] != null ? json['Snow depth (m)'].toDouble() : 0.0,
      totalSnow: json['Total snow (m)'] != null ? json['Total snow (m)'].toDouble() : 0.0,
      cloudCover: json['Cloud cover'] != null ? json['Cloud cover'].toDouble() : 0.0,
      temperature: json['Temperature (t)'] != null ? json['Temperature (t)'].toDouble() : 0.0,
    );
  }
}

class Sensor {
  int sensorID;
  String sensorType;
  bool active;

  Sensor({
    required this.sensorID,
    required this.sensorType,
    required this.active,
  });

  factory Sensor.fromJson(Map<String, dynamic> json) {
    return Sensor(
      sensorID: json['SensorID'],
      sensorType: json['SensorType'] ?? 'nil',
      active: json['Active'],
    );
  }
}
